"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/** The HTML element for highlighting code fragments. */
class HTMLCodeBlockElement extends HTMLElement {
    /**
     * Returns the result of highlighting the received source code string.
     * Before running `customElements.define()`,
     * you need to assign it directly to `HTMLCodeBlockElement.highlight`.
     * @param src - Source code string for highlight
     * @param options - Option for highlight
     * @return - Object of the highlight result
     */
    static highlight = (src, options) => {
        throw new TypeError('The syntax highlighting engine is not set to `HTMLCodeBlockElement.highlight`.');
        return { markup: '' };
    };
    /** Slot elements for Shadow DOM content */
    #slots = (() => {
        /**
         * @param name - The value of name attribute for the slot element
         * @param id - The value of id attribute for the slot element
         * @return - The slot element
         */
        const mkslot = (name, id) => {
            const slot = document.createElement('slot');
            slot.name = name;
            if (id) {
                slot.id = id;
            }
            return slot;
        };
        return {
            name: mkslot('name', 'name'),
            copyButton: mkslot('copy-button'),
            code: mkslot('code'),
        };
    })();
    /** Pure DOM content */
    #a11yName;
    /** Pure DOM content */
    #copyButton;
    /** Pure DOM content */
    #codeBlock;
    /** Pure DOM content */
    #codeWrap;
    /** Actual value of the accessor `value` */
    #value = '';
    /** Actual value of the accessor `label` */
    #label = '';
    /** Actual value of the accessor `language` */
    #language = '';
    /** Actual value of the accessor `controls` */
    #controls = false;
    /** Click event handler of copy button */
    #onClickButton = (() => {
        let key = -1;
        /**
         * Write to the clipboard.
         * @param value - String to be saved to the clipboard
         * @return - A promise
         */
        const copy = (value) => {
            if (navigator.clipboard) {
                return navigator.clipboard.writeText(value);
            }
            return new Promise((r) => {
                const textarea = document.createElement('textarea');
                textarea.value = value;
                document.body.append(textarea);
                textarea.select();
                document.execCommand('copy');
                textarea.remove();
                r();
            });
        };
        return async () => {
            const value = this.#value.replace(/\n$/, '');
            clearTimeout(key);
            await copy(value);
            this.#copyButton.classList.add('--copied');
            this.#copyButton.textContent = 'Copied!';
            key = window.setTimeout(() => {
                this.#copyButton.classList.remove('--copied');
                this.#copyButton.textContent = 'Copy';
            }, 1500);
        };
    })();
    /**
     * Outputs the resulting syntax-highlighted markup to the DOM.
     * @param this - instance
     */
    #render = function () {
        if (!this.parentNode) {
            return;
        }
        const src = (() => {
            if (/[^\n]\n$/.test(this.#value)) {
                return `${this.#value}\n`;
            }
            return this.#value;
        })();
        /** The resulting syntax-highlighted markup */
        const { markup } = HTMLCodeBlockElement.highlight(src, {
            language: this.#language,
        });
        // initialize
        this.textContent = '';
        this.#a11yName.textContent = this.#label;
        this.#slots.name.hidden = !this.#label;
        this.#slots.copyButton.hidden = !this.#controls;
        this.#codeBlock.textContent = '';
        this.#codeBlock.insertAdjacentHTML('afterbegin', markup);
        this.append(this.#a11yName);
        this.append(this.#copyButton);
        this.append(this.#codeWrap);
    };
    /** @return - Syntax Highlighted Source Code */
    get value() {
        return this.#value;
    }
    set value(src) {
        this.#value = String(src);
        this.#render();
    }
    /**
     * The name of code block
     * @return - The value of the label attribute
     */
    get label() {
        return this.#label;
    }
    set label(value) {
        if (value === null) {
            this.#label = '';
            this.removeAttribute('label');
        }
        else {
            this.#label = String(value);
            this.setAttribute('label', this.#label);
        }
        this.#render();
    }
    /**
     * Language Mode
     * @return - The value of the language attribute
     */
    get language() {
        return this.#language;
    }
    set language(value) {
        if (value === null) {
            this.#language = '';
            this.removeAttribute('language');
        }
        else {
            this.#language = String(value);
            this.setAttribute('language', this.#language);
        }
        this.#render();
    }
    /**
     * Flag to display the UI
     * @return - With or without controls attribute
     * */
    get controls() {
        return this.#controls;
    }
    set controls(value) {
        this.#controls = value;
        if (this.#controls) {
            this.setAttribute('controls', '');
        }
        else {
            this.removeAttribute('controls');
        }
        this.#render();
    }
    static get observedAttributes() {
        return [
            'label',
            'language',
            'controls',
        ];
    }
    attributeChangedCallback(attrName, oldValue, newValue) {
        if (oldValue === newValue) {
            return;
        }
        // When the value of the attribute being observed changes,
        // pass value to accessors.
        switch (attrName) {
            // string
            case 'label':
            case 'language':
                this[attrName] = newValue;
                break;
            // boolean
            case 'controls':
                this[attrName] = typeof newValue === 'string';
        }
    }
    connectedCallback() {
        this.#render();
    }
    constructor() {
        super();
        /* -------------------------------------------------------------------------
         * Setup DOM contents
         * ---------------------------------------------------------------------- */
        /** Container of accessible names (label attribute values). */
        const a11yName = (() => {
            const span = document.createElement('span');
            span.slot = 'name';
            span.textContent = this.getAttribute('label') || '';
            return span;
        })();
        const copyButton = (() => {
            const button = document.createElement('button');
            button.type = 'button';
            button.slot = 'copy-button';
            button.textContent = 'Copy';
            button.setAttribute('aria-live', 'polite');
            button.addEventListener('click', this.#onClickButton);
            return button;
        })();
        const codeElm = (() => {
            const code = document.createElement('code');
            code.tabIndex = 0;
            code.className = 'hljs'; // TODO: Make it variable
            return code;
        })();
        const preElm = (() => {
            const pre = document.createElement('pre');
            pre.slot = 'code';
            pre.append(codeElm);
            return pre;
        })();
        /* -------------------------------------------------------------------------
         * Setup Shadow DOM contents
         * ---------------------------------------------------------------------- */
        /**
         * The container of minimum text that will be read even
         * if the accessible name (label attribute value) is omitted.
         */
        const a11yNamePrefix = (() => {
            const span = document.createElement('span');
            span.id = 'semantics';
            span.hidden = true;
            span.textContent = 'Code Block';
            return span;
        })();
        const container = (() => {
            const div = document.createElement('div');
            div.append(...Object.values(this.#slots));
            div.setAttribute('role', 'group');
            div.setAttribute('aria-labelledby', 'semantics name');
            return div;
        })();
        const shadowRoot = this.attachShadow({
            mode: 'closed',
        });
        shadowRoot.append(a11yNamePrefix);
        shadowRoot.append(container);
        /* -------------------------------------------------------------------------
         * Hard private props initialize
         * ---------------------------------------------------------------------- */
        this.#value = (this.textContent || '').replace(/^\n/, '').replace(/\n$/, '');
        this.#label = a11yName.textContent || '';
        this.#language = this.getAttribute('language') || '';
        this.#controls = this.getAttribute('controls') !== null;
        this.#a11yName = a11yName;
        this.#copyButton = copyButton;
        this.#codeBlock = codeElm;
        this.#codeWrap = preElm;
    }
}
exports.default = HTMLCodeBlockElement;
// Protect constructor names from minify
Object.defineProperty(HTMLCodeBlockElement, 'name', {
    value: 'HTMLCodeBlockElement',
});
